<div class="container">
    <ul class="nav nav-tabs">
        <li class="nav-item" class="disabled">
            <a class="nav-link">Connections</a>
        </li>
        <li class="nav-item" class="disabled">
            <a class="nav-link">Variables</a>
        </li>
        <li class="nav-item" class="disabled">
            <a class="nav-link">Pools</a>
        </li>
        <li class="nav-item" class="disabled">
            <a class="nav-link">Environment Variables</a>
        </li>
        <li class="nav-item" class="disabled">
            <a class="nav-link">DAGs cutover</a>
        </li>
    </ul>
</div>
<div class="tab-content" id="tabs-content">
    <div class="container">
        {% block body %}
            <form>
                <table class="table table-striped table-bordered table-hover">
                    <thead>
                    <tr>
                        <th scope="col">Type</th>
                        <th scope="col">Id</th>
                        <th scope="col">Host</th>
                        <th scope="col">Login</th>
                        <th scope="col">Schema</th>
                        <th scope="col">Port</th>
                        <th scope="col">Extra</th>
                        <th>
                        </th>
                    </tr>
                    </thead>
                    <tbody>
                    <tr>
                        <td>Loading....</td>
                        <td></td>
                        <td></td>
                        <td></td>
                        <td></td>
                        <td></td>
                        <td style="font-family: monospace; overflow: hidden; text-overflow: ellipsis;"></td>
                        <td class="text-right">
                            <div>
                                <button class="btn btn-default disabled">Migrate</button>
                            </div>
                        </td>
                    </tr>
                    </tbody>
                </table>
            </form>
        {% endblock %}
    </div>
</div>
