{% extends base_template %}

{% block head_js %}
{{ super() }}
<script src="{{ url_for('starship.static', filename='js/popper.js') }}"></script>
<script src="{{ url_for('starship.static', filename='js/tippy.js') }}"></script>
<script src="{{ url_for('starship.static', filename='js/htmx.min.js') }}"></script>
<script src="{{ url_for('starship.static', filename='js/idiomorph.min.js') }}"></script>
{% endblock %}

{% block head_css %}
{{ super() }}
<style>
    table {
        table-layout: fixed;
    }

    td {
        overflow: hidden;
        text-overflow: ellipsis;
    }
</style>
{% endblock %}

{% block page_title %}
Starship Migration Plugin
{% endblock %}

{% block content %}
<div id="content">
    {% include 'starship/migration.html' %}
</div>
{% endblock %}
