<div class="container" style="padding-top: 15px;">
    <form method="post">
        <input type="hidden" name="csrf_token" value="{{ csrf_token() }}"/>
        <div class="form-group">
            <div class="row">
                <div class="col-lg-3">
                    <label for="currentUser" class="form-label">Current user</label>
                </div>
                <div class="col-lg-6">
                    <label id="currentUser" class="form-label">{{ username }} (<a href="{{ url_for('AstroMigration.logout') }}">Logout</a>)</label>
                </div>
            </div>
            <div class="row" style="padding-top: 15px;">
                <div class="col-lg-3">
                    <label for="selectedAstroDeployment" class="form-label">Target Deployment</label>
                </div>
                <div class="col-lg-6">
                    <select id="selectedAstroDeployment" class="form-control" name="selectedAstroDeployment">
                        <option value="">Select deployment</option>
                        {% if deployments %}
                        {% for deploy in deployments.values() %}
                        <option value="{{deploy.id}}" {{'selected' if deploy.id == session.selectedAstroDeployment}}>
                        {{deploy.label}}
                        </option>
                        {% endfor %}
                        {% endif %}
                    </select>
                </div>
                <div class="col-lg-3">
                    <button type="submit" class="btn btn-primary mb3-l">Select</button>
                </div>
            </div>
        </div>
    </form>
</div>
